@@ -30,7 +30,7 @@ namespace juce
 // This class has been renamed from CoreGraphicsImage to avoid a symbol
 // collision in Pro Tools 2019.12 and possibly 2020 depending on the Pro Tools
 // release schedule.
-class CoreGraphicsPixelData   : public ImagePixelData
+class CoreGraphicsPixelData final : public ImagePixelData
 {
 public:
     CoreGraphicsPixelData (const Image::PixelFormat format, int w, int h, bool clearImage)
@@ -140,7 +140,7 @@ public:
     detail::ContextPtr context;
     detail::ImagePtr cachedImageRef;
 
-    struct ImageDataContainer   : public ReferenceCountedObject
+    struct ImageDataContainer final : public ReferenceCountedObject
     {
         ImageDataContainer() = default;
 
@@ -323,7 +323,7 @@ void CoreGraphicsContext::clipToImageAlpha (const Image& sourceImage, const Affi
         auto image = detail::ImagePtr { CoreGraphicsPixelData::createImage (singleChannelImage, greyColourSpace.get()) };
 
         flip();
-        auto t = AffineTransform::verticalFlip (sourceImage.getHeight()).followedBy (transform);
+        auto t = AffineTransform::verticalFlip ((float) sourceImage.getHeight()).followedBy (transform);
         applyTransform (t);
 
         auto r = convertToCGRect (sourceImage.getBounds());
@@ -472,39 +472,42 @@ void CoreGraphicsContext::fillCGRect (const CGRect& cgRect, bool replaceExisting
         CGContextSetBlendMode (context.get(), kCGBlendModeCopy);
         fillCGRect (cgRect, false);
         CGContextSetBlendMode (context.get(), kCGBlendModeNormal);
+        return;
     }
-    else
+
+    if (state->fillType.isColour())
     {
-        if (state->fillType.isColour())
-        {
-            CGContextFillRect (context.get(), cgRect);
-        }
-        else
-        {
-            ScopedCGContextState scopedState (context.get());
-
-            CGContextClipToRect (context.get(), cgRect);
-
-            if (state->fillType.isGradient())
-                drawGradient();
-            else
-                drawImage (state->fillType.image, state->fillType.transform, true);
-        }
+        CGContextFillRect (context.get(), cgRect);
+        return;
     }
+
+    ScopedCGContextState scopedState (context.get());
+    CGContextClipToRect (context.get(), cgRect);
+
+    if (state->fillType.isGradient())
+        drawGradient();
+    else
+        drawImage (state->fillType.image, state->fillType.transform, true);
 }
 
 void CoreGraphicsContext::fillPath (const Path& path, const AffineTransform& transform)
 {
-    ScopedCGContextState scopedState (context.get());
-
-    setContextClipToPath (path, transform);
-
     if (state->fillType.isColour())
     {
         createPath (path, transform);
-        CGContextFillPath (context.get());
+
+        if (path.isUsingNonZeroWinding())
+            CGContextFillPath (context.get());
+        else
+            CGContextEOFillPath (context.get());
+
+        return;
     }
-    else if (state->fillType.isGradient())
+
+    ScopedCGContextState scopedState (context.get());
+    setContextClipToPath (path, transform);
+
+    if (state->fillType.isGradient())
         drawGradient();
     else
         drawImage (state->fillType.image, state->fillType.transform, true);
@@ -528,7 +531,7 @@ void CoreGraphicsContext::drawImage (const Image& sourceImage, const AffineTrans
     CGContextSetAlpha (context.get(), state->fillType.getOpacity());
 
     flip();
-    applyTransform (AffineTransform::verticalFlip (ih).followedBy (transform));
+    applyTransform (AffineTransform::verticalFlip ((float) ih).followedBy (transform));
     auto imageRect = CGRectMake (0, 0, iw, ih);
 
     if (fillEntireClipAsTiles)
@@ -590,28 +593,25 @@ void CoreGraphicsContext::drawLine (const Line<float>& line)
 
 void CoreGraphicsContext::fillRectList (const RectangleList<float>& list)
 {
-    HeapBlock<CGRect> rects (list.getNumRectangles());
-
-    size_t num = 0;
+    std::vector<CGRect> rects;
+    rects.reserve ((size_t) list.getNumRectangles());
 
     for (auto& r : list)
-        rects[num++] = CGRectMake (r.getX(), flipHeight - r.getBottom(), r.getWidth(), r.getHeight());
+        rects.push_back (CGRectMake (r.getX(), flipHeight - r.getBottom(), r.getWidth(), r.getHeight()));
 
     if (state->fillType.isColour())
     {
-        CGContextFillRects (context.get(), rects, num);
+        CGContextFillRects (context.get(), rects.data(), rects.size());
+        return;
     }
+
+    ScopedCGContextState scopedState (context.get());
+    CGContextClipToRects (context.get(), rects.data(), rects.size());
+
+    if (state->fillType.isGradient())
+        drawGradient();
     else
-    {
-        ScopedCGContextState scopedState (context.get());
-
-        CGContextClipToRects (context.get(), rects, num);
-
-        if (state->fillType.isGradient())
-            drawGradient();
-        else
-            drawImage (state->fillType.image, state->fillType.transform, true);
-    }
+        drawImage (state->fillType.image, state->fillType.transform, true);
 }
 
 void CoreGraphicsContext::setFont (const Font& newFont)
@@ -700,10 +700,8 @@ CoreGraphicsContext::SavedState::SavedState()
 CoreGraphicsContext::SavedState::SavedState (const SavedState& other)
     : fillType (other.fillType), font (other.font), fontRef (other.fontRef),
       textMatrix (other.textMatrix), inverseTextMatrix (other.inverseTextMatrix),
-      gradient (other.gradient.get())
+      gradient (other.gradient.get() != nullptr ? CGGradientRetain (other.gradient.get()) : nullptr)
 {
-    if (gradient != nullptr)
-        CGGradientRetain (gradient.get());
 }
 
 CoreGraphicsContext::SavedState::~SavedState() = default;
@@ -717,9 +715,9 @@ void CoreGraphicsContext::SavedState::setFill (const FillType& newFill)
 static CGGradientRef createGradient (const ColourGradient& g, CGColorSpaceRef colourSpace)
 {
     auto numColours = g.getNumColours();
-    auto data = (CGFloat*) alloca ((size_t) numColours * 5 * sizeof (CGFloat));
-    auto locations = data;
-    auto components = data + numColours;
+    std::vector<CGFloat> data ((size_t) numColours * 5);
+    auto locations = data.data();
+    auto components = locations + numColours;
     auto comps = components;
 
     for (int i = 0; i < numColours; ++i)
@@ -815,7 +813,7 @@ void CoreGraphicsContext::applyTransform (const AffineTransform& transform) cons
 #if USE_COREGRAPHICS_RENDERING && JUCE_USE_COREIMAGE_LOADER
 Image juce_loadWithCoreImage (InputStream& input)
 {
-    struct MemoryBlockHolder   : public ReferenceCountedObject
+    struct MemoryBlockHolder final : public ReferenceCountedObject
     {
         using Ptr = ReferenceCountedObjectPtr<MemoryBlockHolder>;
         MemoryBlock block;
@@ -914,7 +912,7 @@ CGContextRef juce_getImageContext (const Image& image)
      Image retval (Image::ARGB, (int) CGImageGetWidth (image), (int) CGImageGetHeight (image), true);
      CGContextRef ctx = juce_getImageContext (retval);
 
-     CGContextDrawImage (ctx, CGRectMake (0.0f, 0.0f, CGImageGetWidth (image), CGImageGetHeight (image)), image);
+     CGContextDrawImage (ctx, CGRectMake (0.0f, 0.0f, (CGFloat) CGImageGetWidth (image), (CGFloat) CGImageGetHeight (image)), image);
 
      return retval;
  }
